//+------------------------------------------------------------------+
//|                                 TrevA EURUSD                .mq4 |
//|                                                    Trevor Attema |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "TrevA"
#property link      "trevor@attema.name"

//---- input parameters
extern int       StartHour=8;
extern int       ReverseHour=12;
extern int       CloseHour=20;

extern double       positions_each_way = 1.0;
extern double       PercentageOfEquity = 0.4;
extern double       PipValue = 10.0; 
extern int          Stop_Loss = 50;
extern int          TakeProfit = 40;



#define  GO_LONG     1
#define  GO_SHORT    2

#define  FALSE       0
#define  TRUE        1
#define  ERROR       -1

#define EMPTY_HOUR   -1

#define  MAGIC_NUMBER  12031969

int open_hour =EMPTY_HOUR, reverse_hour=EMPTY_HOUR, close_hour=EMPTY_HOUR;


//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
  
   if( Hour() == StartHour && open_hour != Hour())
   {
      if(openOrdersExist() == FALSE)
      {
         double ma = iMA(NULL, 0,200, 0, MODE_EMA,PRICE_CLOSE, 0);
         
         if(Close[0] < ma)
            openOrders( GO_SHORT );
      }
         
      open_hour = Hour();
      close_hour = EMPTY_HOUR;
   }
   
   if( Hour() == ReverseHour && reverse_hour != Hour())
   {
      reverseOrders();
      
      reverse_hour = Hour();
   }
   
   if( Hour() == CloseHour && close_hour != Hour())
   {  
      closeOrders();
      
      open_hour = EMPTY_HOUR;
      reverse_hour = EMPTY_HOUR;   
   }
   
//----
   return(0);
  }
//+------------------------------------------------------------------+

int openOrders( int direction )
{
   //double stop_loss     = 100;
   double lots          = 3;
   double takeProfit    = 0.0;
   int i                = 0;
   
   double stoploss_short = MathAbs(Ask + (Stop_Loss*Point));
   double stoploss_long = MathAbs(Bid - (Stop_Loss*Point));
   
   if( direction == GO_SHORT  )
   {  
   
      lots = getLotSize( positions_each_way, PercentageOfEquity, PipValue, stoploss_short );  
       
      for(i=0; i<positions_each_way; i++)
      {
         if(OrderSend(Symbol(), OP_SELL, lots, Ask , 1, stoploss_short, Ask - (TakeProfit * Point)) == false)
         {
            return(ERROR);         
         }
      }
   }
   
   if( direction == GO_LONG )
   {
      lots = getLotSize( positions_each_way, PercentageOfEquity, PipValue, stoploss_long );  
           
      for(i=0; i<positions_each_way; i++)
      {
         if(OrderSend(Symbol(), OP_BUY, lots, Bid, 1, stoploss_long, Bid + (TakeProfit * Point)) == false)
         {
            return(ERROR);         
         }
      }   

   }
   return(TRUE);
}

void reverseOrders()
{
   double ma = iMA(NULL, 0,200, 0, MODE_EMA,PRICE_CLOSE, 0);
   
   closeOrders();
   
   if(Close[0] > ma)
      openOrders( GO_LONG );
}


#define TICKET_ARRAY_SIZE  40

void closeOrders()
{
   int tickets[TICKET_ARRAY_SIZE];
   int pos = 0;
   
   for(int index = 0; index < TICKET_ARRAY_SIZE; index++)
      tickets[index] = EMPTY_VALUE;

   
   for (int i = 0; i < OrdersTotal( ); i++)
   {
      if(OrderSelect(i, SELECT_BY_POS) == true)
      {
         if(OrderSymbol() == Symbol())
         {
            if( OrderType() == OP_BUY || OrderType() == OP_SELL )
            {
               tickets[pos] = OrderTicket();
               pos++;
           }
         }
      }
   }
       
   for(i = 0; i < TICKET_ARRAY_SIZE; i++)
   {
      if(OrderSelect(tickets[i], SELECT_BY_TICKET) == true)
      {
         if( OrderType() == OP_BUY )
         {
            OrderClose( tickets[i], OrderLots(), Bid, 1);
         }
         
         if( OrderType() == OP_SELL )
         {
            OrderClose( tickets[i], OrderLots(), Ask, 1);
         }
      }
   }

   return(TRUE);
}

int openOrdersExist()
{
   int total = OrdersTotal( );
   
   for (int i=0; i<total; i++)
   {
      if(OrderSelect(i, SELECT_BY_POS) == false)
      {
         return(ERROR);
      }
      
      if(OrderSymbol() == Symbol())
      {
         if( OrderType() == OP_BUY )
         {
            return(TRUE);
         }
         
         if( OrderType() == OP_SELL )
         {
            return(TRUE);
         }         
      }
   }
   
   return(FALSE);
}


double getLotSize( double positions, double riskpercent, double pipvalue, double stoploss )
{
//   stoploss = 50;

  // if( stoploss < 25 )
  //    stoploss = 25;

   double spread     = MarketInfo(Symbol(), MODE_SPREAD); 
//   Print("Spread: ", spread);
   
   double equity     = AccountEquity();
//   Print("Equity: ", equity);
   
   double risk       = equity * ( riskpercent / 100);
//   Print("Risk: ", risk);
//   Print("Stoploss: ", stoploss);
//   Print("Pipvalue: ", pipvalue);
   double lotsize    = risk / (spread + stoploss) / pipvalue;
   Print("Lotsize: ", lotsize);
//   Print("Lotsize%: ", lotsize/positions);
   
   //Print("Lotsize: ", lotsize, " Lot%: ", NormalizeDouble(lotsize/positions, MarketInfo(Symbol(), MODE_DIGITS)));
   
   //return (NormalizeDouble(lotsize/positions, MarketInfo(Symbol(), MODE_DIGITS)));
   return (lotsize/positions);
}